"""
Copyright (c) 2022, 2023 GQYLPY <http://gqylpy.com>. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import os
import sys
import logging

__first__: logging.Logger

gpack = sys.modules[__package__]
gcode = sys.modules[__name__]


def __init__(
        name:    str,
        *,
        level:   str = 'NOTSET',
        output:  str = 'stream',
        logfmt:  str = '[%(asctime)s] [%(module)s.%(funcName)s.line%(lineno)d] '
                       '[%(levelname)s] %(message)s',
        datefmt: str = '%F %T',
        logfile: str = None,
        gname:   str = None
) -> logging.Logger:
    if output.replace(' ', '') not in \
            ("stream", "file", "stream,file", "file,stream"):
        raise TypeError(
            'parameter "output" can only be "stream", "file", or '
            f'"file,stream", not "{output}"'
        )

    logger    = logging.Logger(name, level)
    formatter = logging.Formatter(logfmt, datefmt)

    if 'stream' in output:
        handler = logging.StreamHandler()
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    if 'file' in output:
        if logfile is None:
            starter: str = os.path.basename(sys.argv[0])
            logfile: str = f'/var/log/{starter[:-3]}.log'

        logdir: str = os.path.dirname(os.path.abspath(logfile))
        os.path.isdir(logdir) or os.makedirs(logdir)

        handler = logging.FileHandler(logfile, encoding='UTF-8')
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    if gname:
        if not hasattr(gcode, '__first__') or \
                gcode.__first__.name == 'built-in':
            gcode.__first__ = logger
        setattr(gpack, gname, logger)

    return logger


def __call__(func):
    def inner(
            *msg,
            sep:     str  = ' ',
            oneline: bool = False,
            linesep: str  = '; ',
            gname:   (str, logging.Logger) = None,
            **kw
    ):
        if gname is None:
            if not hasattr(gcode, '__first__'):
                gobj: logging.Logger = __init__(
                    name='built-in',
                    level=gpack.level,
                    output=gpack.output,
                    logfmt=gpack.logfmt,
                    datefmt=gpack.datefmt,
                    **({'logfile': gpack.logfile} if gpack.logfile !=
                        '/var/log/{default is your startup filename}.log'
                       else {}),
                    gname='built-in'
                )
                setattr(gcode, '__first__', gobj)
            gobj: logging.Logger = __first__
        elif gname.__class__ is str:
            gobj: logging.Logger = getattr(gpack, gname, None)
            if gobj.__class__ is not logging.Logger:
                raise NameError(f'gname "{gname}" not found in {__package__}.')
        elif gname.__class__ is logging.Logger:
            gobj: logging.Logger = gname
        else:
            raise TypeError(
                'parameter "gname" type must be "str" or "logging.Logger", '
                f'not "{gname.__class__.__name__}".'
            )

        if sys.version_info >= (3, 8):
            if 'stacklevel' in kw:
                if kw['stacklevel'].__class__ is not int:
                    if not kw['stacklevel'].isdigit():
                        raise TypeError(
                            'parameter "stacklevel" type must be "int", '
                            f'not "{kw["stacklevel"].__class__.__name__}".'
                        )
                    kw['stacklevel'] = int(kw['stacklevel'])
                if kw['stacklevel'] < 2:
                    kw['stacklevel'] = 2
            else:
                kw['stacklevel'] = 2

        msg: str = sep.join(str(m) for m in msg)

        if oneline:
            msg: str = linesep.join(
                m.strip() for m in msg.split('\n') if m and not m.isspace()
            )

        getattr(gobj, func.__name__)(msg, **kw)

    return inner


@__call__
def debug(): ...
@__call__
def info(): ...
@__call__
def warning(): ...
@__call__
def error(): ...
exception = error
@__call__
def critical(): ...
fatal = critical



"""
Secondary encapsulation `logging`, more convenient and fast to create the
logger. Use this module can quickly create instances of `logging.Logger` and
complete a series of log configuration, make your code cleaner.

    >>> import gqylpy_log as glog
    >>> glog.info(...)

    @version: 1.1
    @author: 竹永康 <gqylpy@outlook.com>
    @source: https://github.com/gqylpy/gqylpy-log

────────────────────────────────────────────────────────────────────────────────
Copyright (c) 2022, 2023 GQYLPY <http://gqylpy.com>. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import logging

from typing import TypeVar, Optional, Union, Any
try:
    from typing import Literal
except ImportError:
    Literal = Union

Logger = TypeVar('Logger', str, logging.Logger)

NOTSET   = 'NOTSET'
DEBUG    = 'DEBUG'
INFO     = 'INFO'
WARN     = 'WARN'
WARNING  = 'WARNING'
ERROR    = 'ERROR'
CRITICAL = 'CRITICAL'
FATAL    = 'FATAL'

level   = NOTSET
output  = 'stream'
logfile = '/var/log/{default is your startup filename}.log'
datefmt = '%F %T'
logfmt  = '[%(asctime)s] [%(module)s.%(funcName)s.line%(lineno)d] ' \
          '[%(levelname)s] %(message)s'

OutputMode = Literal['stream', 'file', 'stream,file']


def __init__(
        name:    Optional[str],
        *,
        level:   Optional[str]        = None,
        output:  Optional[OutputMode] = None,
        logfmt:  Optional[str]        = None,
        datefmt: Optional[str]        = None,
        logfile: Optional[str]        = None,
        gname:   Optional[str]        = None
) -> logging.Logger:
    """Get a `logging.Logger` instance.

    @param name:    The name is required and will be passed directly to
                    `logging.Logger`, one string is recommended.
    @param level:   Log level, default is "NOTSET".
    @param output:  Log output mode, stream and/or file, default is "stream".
    @param logfmt:  Passed to `logging.Formatter`, default is "[%(asctime)s]
                    [%(module)s.%(funcName)s.line%(lineno)d] [%(levelname)s]
                    %(message)s".
    @param datefmt: Passed to `logging.Formatter`, default is "%F %T"
    @param logfile: Default is "/var/log/{your startup filename}.log", if the
                    parameter `output` is "file" or "stream,file", otherwise
                    ignore.
    @param gname:   Create a pointer to the `logging.Logger` instance in the
                    `gqylpy_log` module, if not None.
    """


def __getattr__(gname: str) -> logging.Logger:
    """Get the `logging.Logger` instance by `gname`, the `gname` is the
    parameter `gname` passed in `__init__`."""


def debug(
        *msg:    Any,
        sep:     Optional[str]    = None,
        oneline: Optional[bool]   = None,
        linesep: Optional[str]    = None,
        gname:   Optional[Logger] = None,
        **kw
) -> None:
    """
    @param msg:     The log message.
    @param sep:     A string inserted between log message, default a space.
    @param oneline: Make the log message always one line, default is False. If
                    the log message is multi-line, replace the line break with
                    `linesep`.
    @param linesep: A string that replaces the line break, default is "; ". Used
                    with parameter `oneline`.
    @param gname:   An instance of `logging.Logger`, or a pointer name to an
                    instance of `logging.Logger`.
    @param kw:      As a keyword parameter, passed directly to the `debug`
                    method of the `logging.Logger` instance.
    """


def info(
        *msg:    Any,
        sep:     Optional[str]    = None,
        oneline: Optional[bool]   = None,
        linesep: Optional[str]    = None,
        gname:   Optional[Logger] = None,
        **kw
) -> None:
    """
    @param msg:     The log message.
    @param sep:     A string inserted between log message, default a space.
    @param oneline: Make the log message always one line, default is False. If
                    the log message is multi-line, replace the line break with
                    `linesep`.
    @param linesep: A string that replaces the line break, default is "; ". Used
                    with parameter `oneline`.
    @param gname:   An instance of `logging.Logger`, or a pointer name to an
                    instance of `logging.Logger`.
    @param kw:      As a keyword parameter, passed directly to the `info` method
                    of the `logging.Logger` instance.
    """


def warning(
        *msg:    Any,
        sep:     Optional[str]    = None,
        oneline: Optional[bool]   = None,
        linesep: Optional[str]    = None,
        gname:   Optional[Logger] = None,
        **kw
) -> None:
    """
    @param msg:     The log message.
    @param sep:     A string inserted between log message, default a space.
    @param oneline: Make the log message always one line, default is False. If
                    the log message is multi-line, replace the line break with
                    `linesep`.
    @param linesep: A string that replaces the line break, default is "; ". Used
                    with parameter `oneline`.
    @param gname:   An instance of `logging.Logger`, or a pointer name to an
                    instance of `logging.Logger`.
    @param kw:      As a keyword parameter, passed directly to the `warning`
                    method of the `logging.Logger` instance.
    """


def error(
        *msg:    Any,
        sep:     Optional[str]    = None,
        oneline: Optional[bool]   = None,
        linesep: Optional[str]    = None,
        gname:   Optional[Logger] = None,
        **kw
) -> None:
    """
    @param msg:     The log message.
    @param sep:     A string inserted between log message, default a space.
    @param oneline: Make the log message always one line, default is False. If
                    the log message is multi-line, replace the line break with
                    `linesep`.
    @param linesep: A string that replaces the line break, default is "; ". Used
                    with parameter `oneline`.
    @param gname:   An instance of `logging.Logger`, or a pointer name to an
                    instance of `logging.Logger`.
    @param kw:      As a keyword parameter, passed directly to the `error`
                    method of the `logging.Logger` instance.
    """


def exception(
        *msg:    Any,
        sep:     Optional[str]    = None,
        oneline: Optional[bool]   = None,
        linesep: Optional[str]    = None,
        gname:   Optional[Logger] = None,
        **kw
) -> None:
    """
    @param msg:     The log message.
    @param sep:     A string inserted between log message, default a space.
    @param oneline: Make the log message always one line, default is False. If
                    the log message is multi-line, replace the line break with
                    `linesep`.
    @param linesep: A string that replaces the line break, default is "; ". Used
                    with parameter `oneline`.
    @param gname:   An instance of `logging.Logger`, or a pointer name to an
                    instance of `logging.Logger`.
    @param kw:      As a keyword parameter, passed directly to the `error`
                    method of the `logging.Logger` instance.
    """


def critical(
        *msg:    Any,
        sep:     Optional[str]    = None,
        oneline: Optional[bool]   = None,
        linesep: Optional[str]    = None,
        gname:   Optional[Logger] = None,
        **kw
) -> None:
    """
    @param msg:     The log message.
    @param sep:     A string inserted between log message, default a space.
    @param oneline: Make the log message always one line, default is False. If
                    the log message is multi-line, replace the line break with
                    `linesep`.
    @param linesep: A string that replaces the line break, default is "; ". Used
                    with parameter `oneline`.
    @param gname:   An instance of `logging.Logger`, or a pointer name to an
                    instance of `logging.Logger`.
    @param kw:      As a keyword parameter, passed directly to the `critical`
                    method of the `logging.Logger` instance.
    """


def fatal(
        *msg:    Any,
        sep:     Optional[str]    = None,
        oneline: Optional[bool]   = None,
        linesep: Optional[str]    = None,
        gname:   Optional[Logger] = None,
        **kw
) -> None:
    """
    @param msg:     The log message.
    @param sep:     A string inserted between log message, default a space.
    @param oneline: Make the log message always one line, default is False. If
                    the log message is multi-line, replace the line break with
                    `linesep`.
    @param linesep: A string that replaces the line break, default is "; ". Used
                    with parameter `oneline`.
    @param gname:   An instance of `logging.Logger`, or a pointer name to an
                    instance of `logging.Logger`.
    @param kw:      As a keyword parameter, passed directly to the `critical`
                    method of the `logging.Logger` instance.
    """


class _xe6_xad_x8c_xe7_x90_xaa_xe6_x80_xa1_xe7_x8e_xb2_xe8_x90_x8d_xe4_xba_x91:
    gpack = globals()
    gpath = f'{__name__}.g {__name__[7:]}'
    gcode = __import__(gpath, fromlist=...)

    for gname in gpack:
        try:
            assert gname[0] != '_' or gname == '__init__'
            gfunc = getattr(gcode, gname)
            assert gfunc.__module__ in (gpath, __package__)
        except (AssertionError, AttributeError):
            continue
        gfunc.__module__ = __package__
        gfunc.__doc__ = gpack[gname].__doc__
        gpack[gname] = gfunc



import setuptools
import gqylpy_log as g

with open(g.__file__, encoding='utf8') as f:
    for line in f:
        if line.startswith('@version: ', 4):
            version = line.split()[-1]
            break
    author, email = f.readline().split(maxsplit=1)[-1].rstrip().split()
    source = f.readline().split()[-1]

setuptools.setup(
    name=g.__name__,
    version=version,
    author=author,
    author_email=email,
    license='Apache 2.0',
    url='http://gqylpy.com',
    project_urls={'Source': source},
    description='二次封装 logging，更方便快捷的创建日志记录器。使用 gqylpy_log 模块可以快'
                '速创建 logging.Logger 实例并完成一系列的日志配置，使你的代码更简洁。',
    long_description=open('README.md', encoding='utf8').read(),
    long_description_content_type='text/markdown',
    packages=[g.__name__],
    python_requires='>=3.6, <4',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Text Processing :: Indexing',
        'Topic :: Utilities',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11'
    ]
)
